package org.etsi.osl.tmf.gam673.api;


import io.swagger.v3.oas.annotations.Parameter;
import jakarta.validation.Valid;
import org.etsi.osl.tmf.gam673.model.GeographicAddressValidation;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressValidationManagementService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;

@Controller
@RequestMapping("/geographicAddressValidationManagement/v4/")
public class GeographicAddressValidationManagementApiController implements GeographicAddressValidationManagementApi{
    private static final Logger log = LoggerFactory.getLogger(GeographicAddressValidationManagementApiController.class);

    private static final String COULD_NOT_SERIALIZE="Couldn't serialize response for content type application/json";
    private final GeographicAddressValidationManagementService geographicAddressValidationManagementService;

    @Autowired
    public GeographicAddressValidationManagementApiController(GeographicAddressValidationManagementService geographicAddressValidationManagementService) {
        this.geographicAddressValidationManagementService = geographicAddressValidationManagementService;
    }

    //    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<List<GeographicAddressValidation>> listGeographicAddressValidation() {
        try {
            return new ResponseEntity<>(geographicAddressValidationManagementService.findAllGeographicAddressValidations(), HttpStatus.OK);

        } catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddressValidation> createGeographicAddressValidation(
            @Parameter(description = "The geographic address validation to be created", required = true) @Valid @RequestBody GeographicAddressValidation geographicAddressValidation
    ) {
        try{
            GeographicAddressValidation c = geographicAddressValidationManagementService.createGeographicAddressValidation(geographicAddressValidation);
            return new ResponseEntity<>(c, HttpStatus.OK);
        } catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<Void> deleteGeographicAddressValidation(
            @Parameter(description = "Identifier of the geographic address validation", required = true) @PathVariable("id") String id) {

        try {
            return new ResponseEntity<>(geographicAddressValidationManagementService.deleteGeographicAddressValidationById(id), HttpStatus.OK);
        }catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }

    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddressValidation> retrieveGeographicAddressValidation(
            @Parameter(description = "Identifier of the geographic address validation", required = true) @PathVariable("id") String id) {
        try{
            GeographicAddressValidation c = geographicAddressValidationManagementService.findGeographicAddressValidationByUUID(id);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddressValidation> patchGeographicAddressValidation(
            @Parameter(description = "Identifier of the geographic address validation", required = true) @PathVariable("id") String id,
            @Parameter(description = "The Geographic Address Validation to be updated", required = true) @Valid @RequestBody GeographicAddressValidation geographicAddress) {
        try{
            GeographicAddressValidation c = geographicAddressValidationManagementService.updateGeographicAddressValidation(id, geographicAddress);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }



}
