package org.etsi.osl.tmf.gam673.reposervices;

import org.etsi.osl.tmf.gam673.mapper.MapperUtils;
import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicSubAddress;
import org.etsi.osl.tmf.gam673.repo.GeographicAddressManagementRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
@Transactional
public class GeographicAddressManagementService {

    private static final Logger log = LoggerFactory.getLogger(GeographicAddressManagementService.class);

    private final GeographicAddressManagementRepository geographicAddressManagementRepository;

    private MapperUtils mapperUtils;

    @Autowired
    public GeographicAddressManagementService(GeographicAddressManagementRepository geographicAddressManagementRepository) {
        this.geographicAddressManagementRepository = geographicAddressManagementRepository;
    }

    public List<GeographicAddress> findAllGeographicAddress(){
        return (List<GeographicAddress>) geographicAddressManagementRepository.findAll();
    }

    public GeographicAddress findGeographicAddressByUUID(String uuid){
        return geographicAddressManagementRepository.findByUuid(uuid).get();
    }

    public Set<GeographicSubAddress> findAllGeographicSubAddresses(String uuid){
        Optional<GeographicAddress> geoAddress = geographicAddressManagementRepository.findByUuid(uuid);
        return geoAddress.get().getGeographicSubAddresses();
    }

    public GeographicSubAddress findGeographicSubAddressByUUID(String geoAddressId , String geoSubAddressId){
        Optional<GeographicAddress> geoAddress = geographicAddressManagementRepository.findByUuid(geoAddressId);
        return geoAddress.get().getGeographicSubAddresses().stream()
                .filter(geoSubAddress -> geoSubAddress.getUuid().equals(geoSubAddressId))
                .map(geoSubAddress -> geoSubAddress)
                .collect(java.util.stream.Collectors.toList()).get(0);
    }

    public GeographicAddress createGeographicAddress(GeographicAddress geographicAddress){
        log.info("Add another geographic address: {}",geographicAddress);
        return geographicAddressManagementRepository.save(geographicAddress);

    }

    public GeographicAddress updateGeographicAddress(String id,GeographicAddress geographicAddress){
        log.info("Update geographic address with id: {}",id);
        Optional<GeographicAddress> gs = geographicAddressManagementRepository.findByUuid(id);
        return gs.map(address -> updateFields(geographicAddress, address)).orElse(null);
    }

    public Void deleteGeographicAddressById(String id){
        log.info("Delete geographic address with id: {}",id);
        GeographicAddress gs=geographicAddressManagementRepository.findByUuid(id).orElseThrow();
        geographicAddressManagementRepository.delete(gs);
        return null;
    }

    private GeographicAddress updateFields(GeographicAddress newGeographicAddress, GeographicAddress existingGeographicAddress){

        existingGeographicAddress = mapperUtils.geographicAddressMap(newGeographicAddress, existingGeographicAddress);

        geographicAddressManagementRepository.save(existingGeographicAddress);
        return existingGeographicAddress;
    }


}
