package org.etsi.osl.tmf.gam673.reposervices;

import org.etsi.osl.tmf.gam673.mapper.MapperUtils;
import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicAddressValidation;
import org.etsi.osl.tmf.gam673.repo.GeographicAddressValidationManagementRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class GeographicAddressValidationManagementService {

    private static final Logger log = LoggerFactory.getLogger(GeographicAddressValidationManagementService.class);

    private final GeographicAddressValidationManagementRepository geographicAddressValidationManagementRepository;

    private MapperUtils mapperUtils;

    @Autowired
    public GeographicAddressValidationManagementService(GeographicAddressValidationManagementRepository geographicAddressValidationManagementRepository) {
        this.geographicAddressValidationManagementRepository = geographicAddressValidationManagementRepository;
    }

    public List<GeographicAddressValidation> findAllGeographicAddressValidations(){
        return (List<GeographicAddressValidation>) geographicAddressValidationManagementRepository.findAll();
    }

    public GeographicAddressValidation findGeographicAddressValidationByUUID(String uuid){
        return geographicAddressValidationManagementRepository.findByUuid(uuid).get();
    }

    public GeographicAddressValidation createGeographicAddressValidation(GeographicAddressValidation geographicAddressValidation){
        log.info("Add another geographic address validation: {}",geographicAddressValidation);
        return geographicAddressValidationManagementRepository.save(geographicAddressValidation);

    }

    public GeographicAddressValidation updateGeographicAddressValidation(String id,GeographicAddressValidation newGeographicAddressValidation){
        log.info("Update geographic address validation with id: {}",id);
        Optional<GeographicAddressValidation> gs=geographicAddressValidationManagementRepository.findByUuid(id);
        return gs.map(oldAddress -> updateFields(newGeographicAddressValidation, oldAddress)).orElse(null);
    }

    public Void deleteGeographicAddressValidationById(String id){
        log.info("Delete geographic address validation with id: {}",id);
        GeographicAddressValidation gs=geographicAddressValidationManagementRepository.findByUuid(id).orElseThrow();
        geographicAddressValidationManagementRepository.delete(gs);
        return null;
    }

    private GeographicAddressValidation updateFields(GeographicAddressValidation newGeographicAddressValidation, GeographicAddressValidation existingGeographicAddressValidation){

        if(newGeographicAddressValidation.getValidationDate()!=null) existingGeographicAddressValidation.setValidationDate(newGeographicAddressValidation.getValidationDate());
        if(newGeographicAddressValidation.getStatus()!=null) existingGeographicAddressValidation.setStatus(newGeographicAddressValidation.getStatus());
        if(newGeographicAddressValidation.getValidationResult()!=null) existingGeographicAddressValidation.setValidationResult(newGeographicAddressValidation.getValidationResult());
        if(newGeographicAddressValidation.getProvideAlternative()!=null) existingGeographicAddressValidation.setProvideAlternative(newGeographicAddressValidation.getProvideAlternative());

        if(newGeographicAddressValidation.getAlternateGeographicAddresses()!=null){
            for(GeographicAddress n : newGeographicAddressValidation.getAlternateGeographicAddresses()){
                if(n.getUuid()==null){
                    existingGeographicAddressValidation.addGeographicAddress(n);
                }else {
                    for (GeographicAddress oldGeographicAddress : existingGeographicAddressValidation.getAlternateGeographicAddresses()){
                        if (n.getUuid().equals(oldGeographicAddress.getUuid())){
                            GeographicAddress geographicAddress = mapperUtils.geographicAddressMap(n, oldGeographicAddress);
                        }
                    }
                }
            }
        }

        if(newGeographicAddressValidation.getValidAddress()!=null){
            GeographicAddress geographicAddress = mapperUtils.geographicAddressMap(newGeographicAddressValidation.getValidAddress(), existingGeographicAddressValidation.getValidAddress());
            existingGeographicAddressValidation.setValidAddress(geographicAddress);
        }

        geographicAddressValidationManagementRepository.save(existingGeographicAddressValidation);
        return existingGeographicAddressValidation;
    }


}
