package org.etsi.osl.services.api.gam673;

import org.etsi.osl.tmf.gam673.api.GeographicAddressValidationManagementApiController;
import org.etsi.osl.tmf.gam673.model.GeographicAddressValidation;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressValidationManagementService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.when;

class GeographicAddressValidationManagementControllerTest {

    @InjectMocks
    private GeographicAddressValidationManagementApiController controller;


    @Mock
    private GeographicAddressValidationManagementService service;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    void testRetrieveGeographicAddressValidation() {
        List<GeographicAddressValidation> addresses = new ArrayList<>();
        // Add test data to address list
        when(service.findAllGeographicAddressValidations()).thenReturn(addresses);

        ResponseEntity<List<GeographicAddressValidation>> response = controller.listGeographicAddressValidation();

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(addresses, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicAddressValidation(){
        when(service.findAllGeographicAddressValidations()).thenThrow(RuntimeException.class);
        ResponseEntity<List<GeographicAddressValidation>> response = controller.listGeographicAddressValidation();
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }

    @Test
    void testRetrieveGeographicAddressValidationByUUID() {
        GeographicAddressValidation addressValidation = new GeographicAddressValidation();
        // Add test data to address list
        when(service.findGeographicAddressValidationByUUID("Test")).thenReturn(addressValidation);

        ResponseEntity<GeographicAddressValidation> response = controller.retrieveGeographicAddressValidation("Test");

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(addressValidation, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicAddressValidationByUUID(){
        when(service.findGeographicAddressValidationByUUID("test")).thenThrow(RuntimeException.class);
        ResponseEntity<GeographicAddressValidation> response = controller.retrieveGeographicAddressValidation("test");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }


    @Test
    void testCreateGeographicAddressValidation() {
        GeographicAddressValidation address = new GeographicAddressValidation();

        when(service.createGeographicAddressValidation(any())).thenReturn(address);

        ResponseEntity<GeographicAddressValidation> response = controller.createGeographicAddressValidation(address);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(address, response.getBody());
    }

    @Test
    void testExceptionWhenCreateGeographicAddressValidation(){
        GeographicAddressValidation address = new GeographicAddressValidation();

        when(service.createGeographicAddressValidation(any())).thenThrow(RuntimeException.class);

        ResponseEntity<GeographicAddressValidation> response = controller.createGeographicAddressValidation(address);

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

    @Test
    void testDeleteGeographicAddressValidation() {

        doNothing().when(service).deleteGeographicAddressValidationById(anyString());
        ResponseEntity<Void> response = controller.deleteGeographicAddressValidation("addressId");

        assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testDeleteGeographicAddressValidationException() {

        doThrow(RuntimeException.class).when(service).deleteGeographicAddressValidationById(anyString());
        ResponseEntity<Void> response = controller.deleteGeographicAddressValidation("addressId");

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

    @Test
    void testPatchGeographicAddressValidation() {
        String addressId = "addressId";
        GeographicAddressValidation updatedAddress = new GeographicAddressValidation();
        // Set up mock service behavior
        when(service.updateGeographicAddressValidation(anyString(), any())).thenReturn(updatedAddress);

        ResponseEntity<GeographicAddressValidation> response = controller.patchGeographicAddressValidation(addressId, updatedAddress);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(updatedAddress, response.getBody());
    }

    @Test
    void testPatchGeographicAddressValidationException() {
        String addressId = "addressId";
        GeographicAddressValidation updatedAddress = new GeographicAddressValidation();
        // Set up mock service behavior
        when(service.updateGeographicAddressValidation(anyString(), any())).thenThrow(RuntimeException.class);

        ResponseEntity<GeographicAddressValidation> response = controller.patchGeographicAddressValidation(addressId, updatedAddress);

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

}
