package org.etsi.osl.services.api.gam673;

import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicAddressValidation;
import org.etsi.osl.tmf.gam673.repo.GeographicAddressValidationManagementRepository;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressValidationManagementService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;
import java.util.Optional;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@AutoConfigureMockMvc
@ActiveProfiles("testing")
class GeographicAddressValidationManagementServiceTest {

    @InjectMocks
    private GeographicAddressValidationManagementService service;

    @Mock
    private GeographicAddressValidationManagementRepository repository;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    void testFindAllGeographicAddressValidation() {
        // Mock data
        List<GeographicAddressValidation> addresses = List.of(new GeographicAddressValidation(), new GeographicAddressValidation());
        when(repository.findAll()).thenReturn(addresses);

        List<GeographicAddressValidation> result = service.findAllGeographicAddressValidations();

        assertEquals(addresses, result);
    }

    @Test
    void testFindGeographicAddressValidationByUUID() {
        // Mock data
        String uuid = "123";
        GeographicAddressValidation address = new GeographicAddressValidation();
        address.setUuid(uuid);
        when(repository.findByUuid(uuid)).thenReturn(Optional.of(address));

        GeographicAddressValidation result = service.findGeographicAddressValidationByUUID(uuid);

        assertNotNull(result);
        assertEquals(address, result);
    }

    @Test
    void testCreateGeographicAddressValidation() {
        // Mock data
        GeographicAddressValidation address = new GeographicAddressValidation();
        when(repository.save(any())).thenReturn(address);

        GeographicAddressValidation result = service.createGeographicAddressValidation(address);

        assertEquals(address, result);
    }

    @Test
    void testUpdateGeographicAddressValidation() {
        // Mock data
        String id = "123";

        GeographicAddress newAlternativeAddress= new GeographicAddress();
        newAlternativeAddress.setUuid("1234");
        newAlternativeAddress.setLocality("localityNew");

        GeographicAddress oldAlternativeAddress= new GeographicAddress();
        oldAlternativeAddress.setUuid("1234");
        oldAlternativeAddress.setLocality("localityOld");

        GeographicAddressValidation existingAddress = new GeographicAddressValidation();
        existingAddress.setAlternateGeographicAddresses(Set.of(oldAlternativeAddress));

        GeographicAddressValidation newAddress = new GeographicAddressValidation();
        newAddress.setAlternateGeographicAddresses(Set.of(newAlternativeAddress));

        when(repository.findByUuid(id)).thenReturn(Optional.of(existingAddress));
        when(repository.save(any())).thenReturn(existingAddress);

        GeographicAddressValidation result = service.updateGeographicAddressValidation(id, newAddress);

        assertNotNull(result);
        assertEquals(existingAddress, result);
        // Add additional assertions for updated fields if needed
    }

    @Test
    void testDeleteGeographicAddressValidationById() {
        // Mock data
        String id = "123";
        GeographicAddressValidation existingAddress = new GeographicAddressValidation();
        when(repository.findByUuid(id)).thenReturn(Optional.of(existingAddress));

        Void result = service.deleteGeographicAddressValidationById(id);

        assertNull(result);
        verify(repository, times(1)).delete(existingAddress);
    }

}
